/*
 *   SlimProtoLib Copyright (c) 2004,2006 Richard Titmuss
 *
 *   This file is part of SlimProtoLib.
 *
 *   SlimProtoLib is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   SlimProtoLib is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SlimProtoLib; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#include <jni.h>

#include "slimproto/slimproto.h"
#include "slimaudio/slimaudio.h"


JNIEXPORT jint JNICALL Java_org_titmuss_slimprotolib_SlimAudio__1init
  (JNIEnv *env, jobject obj, jint slimproto_ptr) {

  	slimproto_t *p = (slimproto_t *) slimproto_ptr;
  	
  	slimaudio_t *a = (slimaudio_t *) malloc(sizeof(slimaudio_t));  	
  	slimaudio_init(a, p);
  	
  	return (jint) a;  	
}

JNIEXPORT void JNICALL Java_org_titmuss_slimprotolib_SlimAudio__1destroy
  (JNIEnv *env, jobject obj, jint ptr) {
  	
  	slimaudio_t *a = (slimaudio_t *) ptr;
  	slimaudio_destroy(a);
}

JNIEXPORT void JNICALL Java_org_titmuss_slimprotolib_SlimAudio__1open
  (JNIEnv *env, jobject obj, jint ptr) {

  	slimaudio_t *a = (slimaudio_t *) ptr;
	slimaudio_open(a);
}

JNIEXPORT void JNICALL Java_org_titmuss_slimprotolib_SlimAudio__1close
  (JNIEnv *env, jobject obj, jint ptr) {

  	slimaudio_t *a = (slimaudio_t *) ptr;
  	slimaudio_close(a);
}

JNIEXPORT jobjectArray JNICALL Java_org_titmuss_slimprotolib_SlimAudio__1getDevices
  (JNIEnv *env, jobject obj, jint ptr) {
	int i;

/*
slimaudio_debug = true;
slimaudio_decoder_debug = true;
slimaudio_decoder_debug_v = true;
slimaudio_http_debug = true;
slimaudio_http_debug_v = true;
slimaudio_output_debug = true;
*/

  	slimaudio_t *a = (slimaudio_t *) ptr;

	int num_devices;
	char **devices;
	slimaudio_get_output_devices(a, &devices, &num_devices);
	
	jclass stringClazz = (*env)->FindClass(env, "java/lang/String");
	if (stringClazz == NULL) {
		fprintf(stderr, "Cannot load String class\n");
		return NULL;
	}
		
	jobjectArray jBuf = (*env)->NewObjectArray(env, num_devices, stringClazz, NULL);
	if (jBuf == NULL) {
		fprintf(stderr, "Cannot create java object[]\n");
		return NULL;	
	}	
	
	for (i=0; i<num_devices; i++) {
		jstring jDevice = (*env)->NewStringUTF(env, devices[i]);		
		(*env)->SetObjectArrayElement(env, jBuf, i, jDevice);
	}
	
	return jBuf;
}
  
JNIEXPORT void JNICALL Java_org_titmuss_slimprotolib_SlimAudio__1setDevice
  (JNIEnv *evn, jobject obj, jint ptr, jint deviceId) {

  	slimaudio_t *a = (slimaudio_t *) ptr;
  	slimaudio_set_output_device(a, deviceId);
}
