/*
 *   SlimProtoLib Copyright (c) 2004,2006 Richard Titmuss
 *
 *   This file is part of SlimProtoLib.
 *
 *   SlimProtoLib is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   SlimProtoLib is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SlimProtoLib; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include "slimproto/slimproto.h"
#include "slimaudio/slimaudio.h"


static int connect_callback(slimproto_t *p, bool isConnected, void *user_data);
static int parse_macaddress(char *macaddress, const char *str);

slimproto_t slimproto;
slimaudio_t slimaudio;

char macaddress[6] = { 1, 1, 1, 1, 1, 1 };


int main(int argc, char *argv[]) {
	char *slimserver_address = "127.0.0.1";
	int output_device_id = -1;
	char c;
	int i;
	
	while (1) {
		static struct option long_options[] = {
        	{"help", 	no_argument,       0, 'h'},
			{"debug",   required_argument, 0, 'd'},
			{"mac",   	required_argument, 0, 'm'},
			{"output", 	required_argument, 0, 'o'},
	        {0, 0, 0, 0}
	    };
	
	    /* getopt_long stores the option index here. */
	    int option_index = 0;
     
	    c = getopt_long (argc, argv, "hd:m:o:",
    	                 long_options, &option_index);
	
	
		if (c == -1)
			break;
			
		switch (c) {
		case 'h':
			fprintf(stderr, "help ... todo\n");
			exit(0);	
			
		case 'd':
#ifdef SLIMPROTO_DEBUG        
			if (strcmp(optarg, "slimproto") == 0)
				slimproto_debug = true;
			else if (strcmp(optarg, "slimaudio") == 0)
				slimaudio_debug = true;
			else if (strcmp(optarg, "slimaudio_buffer") == 0)
				slimaudio_buffer_debug = true;
			else if (strcmp(optarg, "slimaudio_buffer_v") == 0)
				slimaudio_buffer_debug_v = true;
			else if (strcmp(optarg, "slimaudio_decoder") == 0)
				slimaudio_decoder_debug = true;
			else if (strcmp(optarg, "slimaudio_decoder_v") == 0)
				slimaudio_decoder_debug_v = true;
			else if (strcmp(optarg, "slimaudio_http") == 0)
				slimaudio_http_debug = true;
			else if (strcmp(optarg, "slimaudio_http_v") == 0)
				slimaudio_http_debug_v = true;
			else if (strcmp(optarg, "slimaudio_output") == 0)
				slimaudio_output_debug = true;
			else
				fprintf(stderr, "%s: Unknown debug option %s\n", argv[0], optarg);
			break;
#else
				fprintf(stderr, "%s: Recompilie with -DSLIMPROTO_DEBUG to enable debugging.\n", argv[0]);
#endif

		case 'm':
			if (parse_macaddress(macaddress, optarg) != 0) {
				fprintf(stderr, "%s: Cannot parse mac address %s\n", argv[0], optarg);
				exit(-1);	
			}
			break;
			
		case 'o':
			output_device_id = strtol(optarg, NULL, 0);
			break;
		}
	}

	if (optind < argc)
		slimserver_address = argv[optind];
	
	if (slimproto_init(&slimproto) < 0) {
		fprintf(stderr, "Failed to initialise slimproto\n");
		exit(-1);	
	}
	if (slimaudio_init(&slimaudio, &slimproto) < 0) {
		fprintf(stderr, "Failed to initialise slimaudio\n");
		exit(-1);
	}

	slimproto_add_connect_callback(&slimproto, connect_callback, NULL);

	int num_devices;
	char **devices;
	slimaudio_get_output_devices(&slimaudio, &devices, &num_devices);
	
	printf("Output devices:\n");	
	for (i=0; i<num_devices; i++) {
		printf("%i: %s\n", i, devices[i]);
	}	

	if (output_device_id >= 0) {
		slimaudio_set_output_device(&slimaudio, output_device_id);
	}

	if (slimaudio_open(&slimaudio) < 0) {
		fprintf(stderr, "Failed to open slimaudio\n");
		exit(-1);
	}

	if (slimproto_connect(&slimproto, slimserver_address, 3483) < 0) {
		fprintf(stderr, "Connection to slimserver %s failed.\n", slimserver_address);
		exit(-1);
	}

	getc(stdin);

#if 0
	int ch = getc(stdin);
	// This does not work on all platforms and I don't have time to sort it out right
	// now. getc(stdin) starts reading from the slimproto connection after the user 
	// presses return.
	while (ch != 'q') {
		printf("ch %d %d\n", ch, stdin);
		switch (ch) {
		case 'z': // rewind
			slimproto_ir(&slimproto, 1, 1, 0x7689c03f);
			break;

		case 'b': // forward
			slimproto_ir(&slimproto, 1, 1, 0x7689a05f);
			break;

		case 'c': // pause
			slimproto_ir(&slimproto, 1, 1, 0x768920df);
			break;

		case '+': // volume up
			slimproto_ir(&slimproto, 1, 1, 0x7689807f);
			break;

		case '-': // volume down
			slimproto_ir(&slimproto, 1, 1, 0x768900ff);
			break;

		case 'm': // volume mute
			slimproto_ir(&slimproto, 1, 1, 0x0000c038);
			break;

		case 'x': // play
			slimproto_ir(&slimproto, 1, 1, 0x768910ef);
			break;
		}
		
		ch = getc(stdin);
	}
#endif

	slimaudio_close(&slimaudio);
	slimproto_close(&slimproto);
	
	slimaudio_destroy(&slimaudio);
	slimproto_destroy(&slimproto);
	return 0;
} 

static int connect_callback(slimproto_t *p, bool isConnected, void *user_data) {
	if (isConnected) {
		if (slimproto_helo(&slimproto, 3, 2, (char *)&macaddress, 1, 0) < 0) {
			fprintf(stderr, "Could not send helo to slimserver\n");
			exit(-1);	
		}
	}
	
	return 0;
}

static int parse_macaddress(char *macaddress, const char *str) {
	char *ptr;
	int i;
	
	for (i=0; i<6; i++) {		
		macaddress[i] = (char) strtol (str, &ptr, 16);
		if (i == 5 && *ptr == '\0')
			return 0;
		if (*ptr != ':')
			return -1;
			
		str = ptr+1;
	}	
	return -1;
}
